import pytest
from functools import wraps
from allure_commons.utils import uuid4

from allure.utils import step_parameters
from allure.utils import ALLURE_LABEL_PREFIX, ALLURE_LINK_PREFIX

from allure_commons.constants import Severity
from allure_commons.constants import LabelType, LinkType, AttachmentType


class AllureTestHelper(object):

    def __init__(self, config):
        self.config = config
        self.attach = Attach(config)

    @property
    def attachment_type(self):
        return AttachmentType

    def label(self, label_type, *value):
        allure_label = getattr(pytest.mark, '{prefix}.{label_type}'.format(prefix=ALLURE_LABEL_PREFIX,
                                                                           label_type=label_type))
        return allure_label(*value, label_type=label_type)

    def severity(self, severity):
        return self.label(LabelType.SEVERITY, severity.value)

    @property
    def severity_level(self):
        return Severity

    def feature(self, *features):
        return self.label(LabelType.FEATURE, *features)

    def story(self, *stories):
        return self.label(LabelType.STORY, *stories)

    def link(self, url, link_type=LinkType.LINK, name=None):
        allure_link = getattr(pytest.mark, '{prefix}.{link_type}'.format(prefix=ALLURE_LINK_PREFIX,
                                                                         link_type=link_type))

        pattern = dict(self.config.option.allure_link_pattern).get(str(link_type), '{}')
        url = pattern.format(url)

        return allure_link(url, name=name)

    def issue(self, url, name=None):
        return self.link(url, link_type=LinkType.ISSUE, name=name)

    def testcase(self, url, name=None):
        return self.link(url, link_type=LinkType.TEST_CASE, name=name)

    def step(self, title):
        if callable(title):
            return LazyInitStepContext(self, title.__name__)(title)
        else:
            return LazyInitStepContext(self, title)

    def __getattr__(self, attr):
        for severity in Severity:
            if severity.name == attr:
                return self.severity(severity)

        for attach_type in AttachmentType:
            if attach_type.name == attr:
                return attach_type

        raise AttributeError

    @pytest.hookimpl()
    def pytest_namespace(self):
        return {"allure": self}


class StepContext:

    def __init__(self, allure, title, params):
        self.allure = allure
        self.title = title
        self._uuid = uuid4()
        self.params = params

    def __enter__(self):
        self.allure.config.hook.pytest_allure_before_step(uuid=self._uuid,
                                                          title=self.title,
                                                          params=self.params)

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.allure.config.hook.pytest_allure_after_step(uuid=self._uuid,
                                                         title=self.title,
                                                         exc_type=exc_type,
                                                         exc_val=exc_val,
                                                         exc_tb=exc_tb)

    def __call__(self, func):
        @wraps(func)
        def impl(*a, **kw):
            __tracebackhide__ = True
            params = step_parameters(func, *a, **kw)
            with StepContext(self.allure, self.title.format(*a, **kw), params):
                return func(*a, **kw)
        return impl


class LazyInitStepContext(StepContext):

    def __init__(self, allure_helper, title):
        self.allure_helper = allure_helper
        self.title = title
        self._uuid = uuid4()
        self.params = []

    @property
    def allure(self):
        return self.allure_helper


class Attach(object):

    def __init__(self, config):
        self.config = config

    def __call__(self, body, name=None,  attachment_type=None, extension=None):
        self.config.hook.pytest_allure_attach_data(body=body,
                                                   name=name,
                                                   attachment_type=attachment_type,
                                                   extension=extension)

    def file(self, source, name=None, attachment_type=None, extension=None):
        self.config.hook.pytest_allure_attach_file(source=source,
                                                   name=name,
                                                   attachment_type=attachment_type,
                                                   extension=extension)



import os
import inspect
from itertools import product

from allure_commons.constants import ALLURE_UNIQUE_LABELS


ALLURE_LABEL_PREFIX = 'allure_label'
ALLURE_LINK_PREFIX = 'allure_link'


def allure_parameters(fixturedef, request):
    parameters = {}
    param_name = request.fixturename

    if hasattr(request, 'param'):
        parameters = {'name': fixturedef.ids[request.param_index] if fixturedef.ids else param_name,
                      'value': str(request.param)}

    if 'parametrize' in request.node.keywords.keys():
        param_map = list()
        for mark_info in request.node.keywords['parametrize']:

            _ids = mark_info.kwargs['ids'] if 'ids' in mark_info.kwargs.keys() else None
            _args = mark_info.args[0]
            if not isinstance(_args, (tuple, list)):
                _args = [x.strip() for x in _args.split(",") if x.strip()]

            param_map.append({'args': _args,
                              'has_ids': _ids is not None,
                              'ids': _ids if _ids else mark_info.args[1],
                              'values': list()})

        for variant in product(*[item['ids'] for item in param_map]):
            for i, item in enumerate(param_map):
                item['values'].append(variant[i])

        for item in param_map:
            if param_name in item['args'] and item['has_ids']:
                ids = item['values'][request.param_index]
                if len(item['args']) == 1:
                    parameters = {'name': ids, 'value': str(request.param)}
                else:
                    param_name = '{ids}::{param}'.format(ids=ids, param=param_name)
                    parameters = {'name': param_name, 'value': str(request.param)}

    return parameters


def allure_labels(item):
    for keyword in item.keywords.keys():
        if keyword.startswith(ALLURE_LABEL_PREFIX):
            marker = item.get_marker(keyword)
            label_type = marker.kwargs['label_type']
            if label_type.value in ALLURE_UNIQUE_LABELS:
                yield (label_type.value, marker.args[0])
            else:
                for value in marker.args:
                    yield (label_type.value, value)


def allure_links(item):
    for keyword in item.keywords.keys():
        if keyword.startswith(ALLURE_LINK_PREFIX):
            marker = item.get_marker(keyword)
            link_type = marker.name.split('.', 1)[-1]
            url = marker.args[0]
            name = marker.kwargs['name']
            yield (link_type, url, name)


def allure_package(nodeid):
    parts = nodeid.split('::')
    path = parts[0].split('.')[0]
    return path.replace(os.sep, '.')


def allure_full_name(nodeid):
    parts = nodeid.split('::')
    package = allure_package(nodeid)
    clazz = '.{clazz}'.format(clazz=parts[1]) if len(parts) > 2 else ''
    test = parts[-1]
    return '{package}{clazz}#{test}'.format(package=package, clazz=clazz, test=test)


def step_parameters(func, *a, **kw):
    all_names = inspect.getargspec(func).args
    defaults = inspect.getargspec(func).defaults
    args_part = [(n, str(v)) for n, v in zip(all_names, a)]
    kwarg_part = [(n, str(kw[n]) if n in kw else str(defaults[i])) for i, n in enumerate(all_names[len(a):])]
    return args_part + kwarg_part



import pytest


@pytest.hookspec
def pytest_allure_before_step(uuid, title, params):
    """ called before allure step """


@pytest.hookspec
def pytest_allure_after_step(uuid, title, exc_type, exc_val, exc_tb):
    """ called after allure step """


@pytest.hookspec
def pytest_allure_before_finalizer(parent_uuid, uuid, name):
    """ called before finalizer """


@pytest.hookspec
def pytest_allure_after_finalizer(uuid, exc_type, exc_val, exc_tb):
    """ called after finalizer """


@pytest.hookspec
def pytest_allure_attach_file(source, name, attachment_type, extension):
    """ called for attach file to report """


@pytest.hookspec
def pytest_allure_attach_data(body, name, attachment_type, extension):
    """ called for attach data to report """



from setuptools import setup

PACKAGE = "allure-pytest"
VERSION = "2.0.0b1"

classifiers = [
    'Development Status :: 4 - Beta',
    'Framework :: Pytest',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: Apache Software License',
    'Topic :: Software Development :: Quality Assurance',
    'Topic :: Software Development :: Testing',
]

install_requires = [
    "pytest>=2.7.3",
    "allure-python-commons==2.0.0b1"
]

def main():
    setup(
        name=PACKAGE,
        version=VERSION,
        description="Common module for integrate allure with python-based frameworks",
        url="https://github.com/allure-framework/allure-python2",
        author="QAMetaSoftware, Stanislav Seliverstov",
        author_email="sseliverstov@qameta.io",
        license="Apache-2.0",
        classifiers=classifiers,
        keywords="allure reporting pytest",
        packages=["allure"],
        package_dir={"allure": "src/allure"},
        entry_points={"pytest11": ["allure_pytest = allure.pytest_plugin"]},
        install_requires=install_requires
    )

if __name__ == '__main__':
    main()

